/*
 * SnapShotReader.java
 * Created on 2011/10/13
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

package jp.co.ntt.oss.heapstats.gui.snapshot;

import java.io.Closeable;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Map;

import jp.co.ntt.oss.heapstats.container.ObjectData;
import jp.co.ntt.oss.heapstats.container.SnapShot;
import jp.co.ntt.oss.heapstats.container.SnapShotHeader;

/**
 * This class will load the temporary files java heap information.
 */
public class SnapShotReader implements
                           Closeable {
    /** the temporary files java heap information. */
    private ObjectInputStream stream;

    /** the temporary files java heap information. */
    private FileChannel streamChannel;

    /**
     * Create a SnapShotReader.
     *
     * @param fname the temporary file path java heap information.
     * @throws IOException if an I/O error occurs while reading stream header
     */
    public SnapShotReader(final String fname) throws IOException {
        FileInputStream fstream = new FileInputStream(fname);
        streamChannel = fstream.getChannel();
        stream = new ObjectInputStream(fstream);
    }

    /**
     * Returns the header part of the specified snapshot.
     *
     * @param offset the channel's file position.
     * @return the SnapShot header.
     * @throws IOException If an I/O error has occurred.
     * @throws ClassNotFoundException Class of a serialized object cannot be
     *         found.
     */
    private final SnapShotHeader getHeader(final long offset)
            throws IOException, ClassNotFoundException {
        streamChannel.position(offset);
        return (SnapShotHeader) stream.readObject();
    }

    /**
     * Return the snapshot header and objects data.
     *
     * @param offset the channel's file position.
     * @return Return the SnapShot information.
     * @throws IOException If an I/O error has occurred.
     * @throws ClassNotFoundException Class of a serialized object cannot be
     *         found.
     */
    public final synchronized SnapShot getSnapShot(final long offset)
            throws IOException, ClassNotFoundException {
        SnapShot snapShot = new SnapShot();

        snapShot.setHeader(getHeader(offset));
        Map<Long, ObjectData> elements = new HashMap<Long, ObjectData>();
        ObjectData data;
        long heapUsage = 0;

        for (int i = 0; i < snapShot.getHeader().getNumEntries(); i++) {
            data = (ObjectData) stream.readObject();
            elements.put(data.getTag(), data);
            heapUsage += data.getTotalSize();
        }

        snapShot.setElements(elements);
        snapShot.setHeapUsage(heapUsage);

        return snapShot;
    }

    /**
     * Close the input stream.
     *
     * @throws IOException If an I/O error has occurred.
     */
    public final void close() throws IOException {
        streamChannel.close();
        stream.close();
    }
}
